import { Icon } from '@iconify/react/dist/iconify.js';
import cx from 'clsx';
import { useAtom, useAtomValue } from 'jotai';
import { FC, useCallback, useMemo, useState } from 'react';
import { useDebounce } from 'react-use';
import { ScrollArea } from '../../components/ScrollArea';
import { CurrentPatternAtom, Pattern, PatternsAtom, totalDuration } from '../../context/Patterns';
import styles from './Patterns.module.css';

const PatternCard: FC<{ pattern: Pattern }> = ({ pattern }) => {
  const [currentPattern, setCurrentPattern] = useAtom(CurrentPatternAtom);
  const duration = useMemo(() => {
    return totalDuration(pattern) / 1000;
  }, [pattern]);
  const selected = useMemo(() => currentPattern?.id === pattern.id, [currentPattern, pattern]);
  const handleSingleClick = useCallback(() => {
    if (currentPattern?.id === pattern.id) {
      setCurrentPattern(null);
    } else {
      setCurrentPattern(pattern);
    }
  }, [pattern, currentPattern]);

  return (
    <div
      className={cx(styles.pattern_card, selected && styles.selected)}
      onClick={handleSingleClick}>
      <h6 className={styles.name}>{pattern.name}</h6>
      <span>{duration.toFixed(2)} s</span>
    </div>
  );
};

const Patterns: FC = () => {
  const [rawKeyword, setRawKeyword] = useState<string>('');
  const [keyword, setKeyword] = useState<string | null>(null);
  const patterns = useAtomValue(PatternsAtom(keyword));

  useDebounce(
    () => {
      if (rawKeyword.trim().length > 0) {
        setKeyword(rawKeyword.trim());
      } else {
        setKeyword(null);
      }
    },
    1500,
    [rawKeyword],
  );

  return (
    <div className={styles.patterns}>
      <div className={styles.search_row}>
        <div className={cx('input_wrapper', 'extendable')}>
          <Icon icon="material-symbols-light:search" />
          <input
            type="text"
            placeholder="Search patterns..."
            value={rawKeyword}
            onChange={(evt) => setRawKeyword(evt.currentTarget.value)}
          />
        </div>
        <button className="tonal secondary">
          <Icon icon="material-symbols-light:add" />
          New Pattern
        </button>
      </div>
      <ScrollArea enableY>
        <div className={styles.pattern_list}>
          {patterns.length === 0 && <div className="empty_prompt">No pattern found.</div>}
          {patterns.map((p) => (
            <PatternCard key={p.id} pattern={p} />
          ))}
        </div>
      </ScrollArea>
    </div>
  );
};

export default Patterns;
